<?php
/**
 * Widget Generator
 * 
 * @package FEEDPRESS_GENRATOR_Feed_Widget_Generator/includes
 * @since 1.0
 */
class FEEDPRESS_GENERATOR_FWG_Generator
{
	/**
	 * Main Instance
	 * 
	 * @since 1.0
	 * @var FEEDPRESS_GENERATOR_FWG_Generator
	 */
	protected static $_instance = null;

	/**
	 * Plugin Settings
	 * 
	 * @since 1.0
	 * @var array
	 */
	var $plugin_settings;

	/**
	 * Widget options fields
	 * 
	 * @since 1.0
	 * @var array
	 */
	var $options_fields;

	/**
	 * Widget options sections
	 * 
	 * @since 1.0
	 * @var array
	 */
	var $options_sections;

	/**
	 * Sanitized user defined widget settings
	 * 
	 * @since 1.0
	 * @var array
	 */
	protected static $_user_sanitized_settings = null;

	/**
	 * Constructor
	 * 
	 * @since 1.0
	 * 
	 * @param array $plugin_settings
	 * @return FEEDPRESS_GENERATOR_FWG_Generator
	 */
	public function __construct( array $plugin_settings )
	{
		// setups
		$this->plugin_settings = $plugin_settings;
		$this->get_options_fields();
		$this->get_options_sections();
	}

	/**
	 * Widget Output endpoint
	 * 
	 * @return void
	 */
	public static function output_endpoint()
	{
		global $wp_query;

		// skip unrelated requests
		if ( 'feeds-widget' !== $wp_query->get( 'name' ) || !$wp_query->is_404 )
			return;

		// prevent 404 page
		$wp_query->is_404 = false;
		$wp_query->is_archive = true;
		header( 'HTTP/1.1 200 OK' );

		// holders
		$generator = self::instance();
		$settings = $generator->get_widget_settings();
		wp_localize_script( 'rss-fwg-widget', 'feedpress_generator_feed_settings', $settings );

		// widget header
		require apply_filters( 'feedpress_generator_fwg_widget_template_header', FEEDPRESS_GENERATOR_FWG_DIR . 'views/widget/header.php' );

		// widget content
		require apply_filters( 'feedpress_generator_fwg_widget_template_content', FEEDPRESS_GENERATOR_FWG_DIR . 'views/widget/content.php' );

		// widget footer
		require apply_filters( 'feedpress_generator_fwg_widget_template_footer', FEEDPRESS_GENERATOR_FWG_DIR . 'views/widget/footer.php' );

		// That's all, folks!
		die();
	}

	public function fetch_feeds()
	{
		$settings = $this->get_widget_settings();

		// check empty feeds
		if ( empty( $settings['feeds'] ) )
		{
			/**
			 * Filters fetched object to render
			 * 
			 * @since 1.0.0
			 * 
			 * @param array|WP_Error $feeds_result
			 * @param array $settings
			 */
			return apply_filters( 'feedpress_generator_fwg_widget_feeds_fetch', new WP_Error( 'feed', __( 'Empty Feed URLs', FEEDPRESS_FWG_DOMAIN ) ), $settings );
		}

		// feed loop
		$feeds_result = array();
		foreach ( $settings['feeds'] as $feed_index => $feed_url )
		{
			$feed = fetch_feed( $feed_url );

			// check if feed is valid
			if ( is_wp_error( $feed ) )
				continue;

			// build results
			$feeds_result[ $feed_index ] = array ( 
					'url' => $feed_url,
					'title' => $feed->get_title(),
			);

			$fetched_items = $feed->get_items( 0, $feed->get_item_quantity( $settings['entry_limit'] ) );
			$feed_items = array();

			// feed items loop
			/* @var $item SimplePie_Item */
			foreach ( $fetched_items as $index => $item )
			{
				$feed_items[] = array ( 
						'title' => $item->get_title(),
						'url' => $item->get_permalink(),
						'content' => $item->get_content(),
						'datetime' => $item->get_date( 'Y-m-d H:i:s' ),
						'object' => $item,
				);
			}

			$feeds_result[ $feed_index ]['items'] = $feed_items;
			unset( $feed_items, $fetched_items );
		}

		/**
		 * Filters fetched object to render
		 *
		 * @since 1.0.0
		 *
		 * @param array|WP_Error $feeds_result
		 * @param array $settings
		 */
		return apply_filters( 'feedpress_generator_fwg_widget_feeds_fetch', $feeds_result, $settings );
	}

	/**
	 * Get widget settings default values
	 * 
	 * @return array
	 */
	public function get_settings_defaults()
	{
		return array_map( function( $field_args ) {
			return isset( $field_args['default'] ) ? $field_args['default'] : '';
		}, $this->options_fields );
	}

	/**
	 * Get user defined widget settings/options
	 * 
	 * @param array $settings (optional)
	 * @return array
	 */
	public function get_widget_settings( $user_settings = null )
	{
		if ( is_null( self::$_user_sanitized_settings ) || !is_null( $user_settings ) )
		{
			if ( is_null( $user_settings ) )
				$user_settings = $_GET;

			// default values
			$defaults = $this->get_settings_defaults();
			$user_settings = wp_parse_args( $user_settings, $defaults );

			foreach ( $user_settings as $option_name => &$option_value )
			{
				// get option/field args
				$option_args = $this->get_options_fields( $option_name );

				// skip undefined options
				if ( !$option_args )
				{
					unset( $user_settings[ $option_name ] );
					continue;
				}

				// set default value of empty
				if ( empty( $option_value ) )
					$option_value = $defaults[ $option_name ];

				// data type sanitize
				switch ( $option_args['data_type'] )
				{
					case 'url':
						$sanitize_callback = 'esc_url';
						break;

					case 'number':
						$sanitize_callback = 'absint';
						break;

					case 'color':
						$sanitize_callback = 'FEEDPRESS_GENERATOR_FWG_Helpers::sanitize_hex_color';
						break;

					default:
						$sanitize_callback = 'sanitize_text_field';
				}

				// apply sanitizing
				if ( is_array( $option_value ) )
					$option_value = array_map( $sanitize_callback, $option_value );
				else
					$option_value = call_user_func( $sanitize_callback, $option_value );

				// validation based on input requirements
				switch ( $option_name )
				{
					case 'feeds':
						if ( !is_array( $option_value ) )
							$option_value = array( $option_value );

						// filter out empty values
						$option_value = array_filter( $option_value, function( $val ) {
							return !empty( $val );
						} );
						break;

					case 'feed_title_bg_image':
					case 'entry_title_bg_image':
					case 'footer_bg_image':
						if ( !empty( $option_value ) && !getimagesize( $option_value ) )
							$option_value = $defaults[ $option_name ];
						break;
				}
			}

			self::$_user_sanitized_settings = apply_filters( 'feedpress_generator_fwg_generator_widget_settings', $user_settings );
		}

		return self::$_user_sanitized_settings;
	}

	/**
	 * Get options fields for that generator based on plugin settings
	 * 
	 * @since 1.0
	 * 
	 * @param boolean $basic_options_only
	 * @return array
	 */
	public function get_available_fields( $basic_options_only = null )
	{
		$default_args = $this->default_field_args();

		// parse default field args after filter the fields
		return array_map( function( $field_args ) use ( $default_args ) {
			return wp_parse_args( $field_args, $default_args );
		} , array_filter( $this->get_options_fields(), $this->options_fields_filter( $basic_options_only ) ) );
	}

	/**
	 * Widget output script URI
	 * 
	 * @since 1.0
	 * 
	 * @return string
	 */
	public function get_widget_script_uri()
	{
		return FEEDPRESS_GENERATOR_FWG_URI .'assets/js/widget'. FEEDPRESS_GENERATOR_FWG_Helpers::enqueue_suffix() .'.js';
	}

	/**
	 * Options fields filter lambda callback
	 * 
	 * @param boolean $basic_options_only
	 * @return callable
	 */
	public function options_fields_filter( $basic_options_only = null )
	{
		// wither to use basic options only or all
		if ( $basic_options_only === null )
			$basic_options_only = 'basic' === feedpress_generator_fwg_get_settings( 'enable_settings' );

		return function ( $field ) use ( $basic_options_only ) {
			// pass basic fields only
			if ( $basic_options_only )
				return 'basic' === $field[ 'option_type' ];

			// otherwise pass all fields
			return true;
		};
	}

	/**
	 * Widget options fields
	 * 
	 * @since 1.0
	 * 
	 * @param string $field_name
	 * @return array
	 */
	public function get_options_fields( $field_name = '' )
	{
		if ( is_null( $this->options_fields ) )
		{
			/**
			 * NOTE: if you are going to override those fields, make sure you order them
			 * based on sections, each section fields are grouped together.
			 */
			$this->options_fields = apply_filters( 'feedpress_generator_fwg_generator_options_fields', array ( 
					/*
					 * Feeds URLs
					 */
					'feeds' => array (
							'label' => __( 'Feeds URLs:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'feeds',
							'option_type' => 'basic',
							'field_order' => 1,
							'input_type' => 'text',
							'data_type' => 'url',
							'default' => feedpress_generator_fwg_get_settings( 'default_feedpress_generator_feed' ),
							'attributes' => array ( 
									'class' => array( 'large-text', 'code' ),
							),
					),
					/*
					 * Customizations
					 */
					'width' => array (
							'label' => __( 'Width:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 3,
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '300',
							'attributes' => array ( 
									'class' => array( 'small-text', 'code' ),
									'min' => 0,
									'step' => 1,
							),
							'description' => 'px',
					),
					'height' => array (
							'label' => __( 'Height:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 4,
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '300',
							'attributes' => array ( 
									'class' => array( 'small-text', 'code' ),
									'min' => 0,
									'step' => 1,
							),
							'description' => 'px',
					),
					'responsive_width' => array (
							'label' => __( 'Responsive Widget:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 5,
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'widget_id' => array (
							'label' => __( 'Widget ID:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'text',
							'default' => 'rss-feeds-widget',
							'attributes' => array (
									'class' => array( 'large-text', 'code' ),
							),
					),
					'auto_scroll_enable' => array (
							'label' => __( 'Auto scroll:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 6,
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'on',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'scroll_step' => array (
							'label' => __( 'Scroll step:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '6',
							'attributes' => array ( 
									'class' => array( 'small-text', 'code' ),
									'min' => 1,
									'step' => 1,
							),
							'description' => __( 'Seconds', FEEDPRESS_FWG_DOMAIN ),
					),
					'scroll_bar_enable' => array (
							'label' => __( 'Widget with scroll bar:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'scroll_bar_dir' => array (
							'label' => __( 'Scroll bar direction:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 7,
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'vertical',
							'options' => array( 'vertical' => __( 'Vertical', FEEDPRESS_FWG_DOMAIN ), 'horizontal' => __( 'Horizontal', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'link_target' => array (
							'label' => __( 'Link open', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'new',
							'options' => array( 'new' => __( 'New Window', FEEDPRESS_FWG_DOMAIN ), 'same' => __( 'Same window', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'font_face' => array (
							'label' => __( 'Font face:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 8,
							'input_type' => 'select',
							'data_type' => 'text',
							'options' => array ( 
									'Arial' => 'Arial',
									'Verdana' => 'Verdana',
									'Tahoma' => 'Tahoma',
									'Times New Roman' => 'Times New Roman',
							),
					),
					'font_size' => array ( 
							'label' => __( 'Font size:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '12',
							'attributes' => array (
									'class' => array( 'small-text', 'code' ),
									'min' => 1,
									'step' => 1,
							),
							'description' => 'px',
					),
					'border_enable' => array ( 
							'label' => __( 'Border:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'on',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'border_color' => array ( 
							'label' => __( 'Border Color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 9,
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#cccccc',
					),
					'source_icon_enable' => array ( 
							'label' => __( 'Source icon:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'entry_limit' => array ( 
							'label' => __( 'Number of posts:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 10,
							'input_type' => 'number',
							'data_type' => 'number',
							'field_order' => 2,
							'default' => '0',
							'attributes' => array (
									'class' => array( 'small-text', 'code' ),
									'disabled' => 'disabled',
									'min' => 0,
									'step' => 1,
							),
							'description' => '<label><input type="checkbox" id="entry-limit-show-all" data-previous-value="5" '.
													'checked="checked" /> '. __( 'Show All', FEEDPRESS_FWG_DOMAIN ) .'</label>',
					),
					'custom_css' => array ( 
							'label' => __( 'URL of custom CSS:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'url',
							'attributes' => array (
									'class' => array( 'large-text', 'code' ),
							),
					),
					'custom_css_examples' => array ( 
							'label' => __( 'Custom CSS examples:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'customize',
							'option_type' => 'basic',
							'field_order' => 11,
							'input_type' => 'radio',
							'data_type' => 'text',
							'options' => array ( 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style1.css' => '1', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style2.css' => '2', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style3.css' => '3', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style4.css' => '4', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style5.css' => '5', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style6.css' => '6', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style7.css' => '7', 
									FEEDPRESS_GENERATOR_FWG_URI .'assets/css-examples/style8.css' => '8', 
							),
							'description' => '<a href="javascript:void(0)" class="css-example-clear">'. __( 'Clear Example Style', FEEDPRESS_FWG_DOMAIN ) .'</a>',
					),
					/*
					 * Widget Main title
					 */
					'feed_title_enable' => array (
							'label' => __( 'Feed title:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'on',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'feed_bg_color' => array (
							'label' => __( 'Feed background color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'basic',
							'field_order' => 12,
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#ffffff',
					),
					'feed_title_bg_color' => array (
							'label' => __( 'Choose background color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#3366ff',
					),
					'feed_title_bg_image' => array (
							'label' => __( 'Background image:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'url',
							'attributes' => array (
									'class' => array( 'large-text', 'code' ),
							),
					),
					'feed_title_color' => array (
							'label' => __( 'Choose font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#ffffff',
					),
					'feed_title_text' => array (
							'label' => __( 'Feed title:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'main_title', 
							'option_type' => 'basic',
							'field_order' => 13,
							'input_type' => 'text',
							'data_type' => 'text',
							'attributes' => array (
									'class' => array( 'large-text' ),
							),
					),
					/*
					 * Entry title
					 */
					'entry_title_length' => array (
							'label' => __( 'Maximum length:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_title',
							'option_type' => 'advanced',
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '50',
							'attributes' => array (
									'class' => array( 'small-text', 'code' ),
									'min' => 1,
									'step' => 1,
							),
					),
					'entry_title_color' => array (
							'label' => __( 'Choose font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_title',
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#666666',
					),
					'entry_title_bg_color' => array (
							'label' => __( 'Choose background color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_title',
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#ffffff',
					),
					'entry_title_bg_image' => array (
							'label' => __( 'Background image:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_title',
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'url',
							'attributes' => array (
									'class' => array( 'large-text', 'code' ),
							),
					),
					'entry_separator_line_enable' => array (
							'label' => __( 'Separator line:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_title',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'on',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					/*
					 * Entry Content
					 */
					'entry_content_enable' => array ( 
							'label' => __( 'Contents:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'basic',
							'field_order' => 14,
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'on',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'entry_content_length' => array (
							'label' => __( 'Maximum length:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'advanced',
							'input_type' => 'number',
							'data_type' => 'number',
							'default' => '120',
							'attributes' => array (
									'class' => array( 'small-text', 'code' ),
									'min' => 1,
									'step' => 1,
							),
					),
					'entry_content_ago_enable' => array ( 
							'label' => __( 'Posted time ago:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'entry_content_color' => array (
							'label' => __( 'Content font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'basic',
							'field_order' => 15,
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#666666',
					),
					'entry_content_html_enable' => array ( 
							'label' => __( 'HTML (images / links):', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'advanced',
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'entry_content_link_color' => array (
							'label' => __( 'Links font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'entry_content',
							'option_type' => 'basic',
							'field_order' => 16,
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#333333',
					),
					/*
					 * Footer
					 */
					'footer_enable' => array (
							'label' => __( 'Feed footer:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'footer',
							'option_type' => 'basic',
							'field_order' => 17,
							'input_type' => 'radio',
							'data_type' => 'text',
							'default' => 'off',
							'options' => array( 'on' => __( 'On', FEEDPRESS_FWG_DOMAIN ), 'off' => __( 'Off', FEEDPRESS_FWG_DOMAIN ) ),
					),
					'footer_bg_color' => array (
							'label' => __( 'Background color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'footer',
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#ffffff',
					),
					'footer_font_color' => array (
							'label' => __( 'Choose font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'footer',
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#333333',
					),
					'footer_bg_image' => array (
							'label' => __( 'Background image:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'footer',
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'url',
							'attributes' => array (
									'class' => array( 'large-text', 'code' ),
							),
					),
					'footer_text' => array (
							'label' => __( 'Footer name:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'footer',
							'option_type' => 'advanced',
							'input_type' => 'text',
							'data_type' => 'text',
							'default' => __( 'RSS feed', FEEDPRESS_FWG_DOMAIN ),
							'attributes' => array (
									'class' => array( 'large-text' ),
							),
					),
					/*
					 * Bottom link
					 */
					'bottom_link_color' => array ( 
							'label' => __( 'Choose font color:', FEEDPRESS_FWG_DOMAIN ),
							'section' => 'bottom_link',
							'option_type' => 'advanced',
							'input_type' => 'color',
							'data_type' => 'color',
							'default' => '#cccccc',
							'description' => __( 'Caution: please be aware that search engines do not like links to be the same color as the background of the page.', FEEDPRESS_FWG_DOMAIN ),
					),
			) );
		}

		return !empty( $field_name ) && isset( $this->options_fields[$field_name] ) ? $this->options_fields[$field_name] : $this->options_fields;
	}

	/**
	 * Widget options sections
	 * 
	 * @param string $section_name (optional)
	 * @return array|null if <code>$section_name</code> is set will return array of that section arguments otherwise returns list of registered sections. 
	 */
	public function get_options_sections( $section_name = '' )
	{
		if ( is_null( $this->options_sections ) )
		{
			$this->options_sections =  apply_filters( 'feedpress_generator_fwg_generator_options_sections', array ( 
					'feeds' => array ( 
							'title' => __( 'Add Feeds', FEEDPRESS_FWG_DOMAIN ),
							'subtitle' => __( 'Add the feed or blog url', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Add up to 5 feed or page URLs here. Try to use the direct feed URL if known.', FEEDPRESS_FWG_DOMAIN ),
					),
					'customize' => array ( 
							'title' => __( 'Customize your Feed Widget', FEEDPRESS_FWG_DOMAIN ),
							'subtitle' => __( 'General settings', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Specify the feed size, autoscroll and other general settings.', FEEDPRESS_FWG_DOMAIN ),
					),
					'main_title' => array ( 
							'title' => __( 'Main title', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Settings and display for the feed widget title.', FEEDPRESS_FWG_DOMAIN ),
					),
					'entry_title' => array ( 
							'title' => __( 'Entry title', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Settings and display for the title of the post / item, and specify the main background color.', FEEDPRESS_FWG_DOMAIN ),
					),
					'entry_content' => array ( 
							'title' => __( 'Entry contents', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Settings and display for the post / item content.', FEEDPRESS_FWG_DOMAIN ),
					),
					'footer' => array ( 
							'title' => __( 'Feed footer', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Settings and display for the feed widget footer.', FEEDPRESS_FWG_DOMAIN ),
					),
					'bottom_link' => array ( 
							'title' => __( 'Bottom Link', FEEDPRESS_FWG_DOMAIN ),
							'description' => __( 'Style the widget backlink.', FEEDPRESS_FWG_DOMAIN ),
					),
			) );
			
			
			

		}
		
		

		// return specific section information if is set
		if ( !empty( $section_name ) )
		{
			return isset( $this->options_sections[ $section_name ] ) ? wp_parse_args( $this->options_sections[ $section_name ],array ( 
					'title' => '',
					'subtitle' => '',
					'description' => '',
			) ) : null;
		}

		// otherwise return all sections
		return $this->options_sections;
	}

	/**
	 * Default options field args
	 * 
	 * @since 1.0
	 * 
	 * @return array
	 */
	public function default_field_args()
	{
		return array ( 
				'label' => '',
				'section' => 'feeds',
				'option_type' => 'basic',
				'input_type' => 'text',
				'data_type' => 'text',
				'field_order' => 0,
				'default' => '',
				'attributes' => array(),
				'description' => '',
		);
	}

	/**
	 * Main FEEDPRESS_GENERATOR_FWG_Generator Instance
	 *
	 * Ensures only one instance of FEEDPRESS_GENERATOR_FWG_Generator is loaded or can be loaded.
	 *
	 * @since 1.0
	 * @static
	 *
	 * @return FEEDPRESS_GENERATOR_FWG_Generator
	 */
	public static function instance()
	{
		if ( is_null( self::$_instance ) )
			self::$_instance = new self( FEEDPRESS_GENERATOR_FEED_WIDGET()->backend->get_settings() );
	
		return self::$_instance;
	}
}




