<?php
/**
 * Help functions
 * 
 * @package FEEDPRESS_GENRATOR_Feed_Widget_Generator/includes
 * @since 1.0
 */

/**
 * Sanitize CSS
 * 
 * @param string $input_css
 * @return string
 */
function feedpress_generator_fwg_sanitize_css( $input_css )
{
	// HTMLPurifier lib
	if ( !class_exists( 'HTMLPurifier' ) )
		require_once FEEDPRESS_GENERATOR_FWG_DIR . 'includes/htmlpurifier/HTMLPurifier.auto.php';

	// CSSTidy lib
	if ( !class_exists( 'csstidy' ) )
		require_once FEEDPRESS_GENERATOR_FWG_DIR . 'includes/csstidy/class.csstidy.php';

	// Create a new configuration object
	$config = HTMLPurifier_Config::createDefault();
	$config->set( 'Filter.ExtractStyleBlocks', true );

	// Create a new purifier instance
	$purifier = new HTMLPurifier( $config );

	// Wrap our CSS in style tags and pass to purifier.
	// we're not actually interested in the html response though
	$html = $purifier->purify( '<style>'.$input_css.'</style>' );

	// Revert error reporting
	error_reporting($level);

	// The "style" blocks are stored seperately
	$output_css = $purifier->context->get( 'StyleBlocks' );

	// Get the first style block
	return $output_css[0];
}

/**
 * Helper methods
 * 
 * @package FEEDPRESS_GENRATOR_Feed_Widget_Generator/includes
 * @since 1.0
 */
final class FEEDPRESS_GENERATOR_FWG_Helpers
{
	/**
	 * Text Domain
	 * 
	 * @var string
	 */
	static $text_domain = FEEDPRESS_FWG_DOMAIN;

	/**
	 * Sanitizes a hex color.
	 *
	 * Returns either '', a 3 or 6 digit hex color (with #), or null.
	 * For sanitizing values without a #, see self::sanitize_hex_color_no_hash().
	 *
	 * @since 1.0
	 *
	 * @param string $color
	 * @return string|null
	 */
	public static function sanitize_hex_color( $color ) 
	{
		if ( '' === $color )
			return '';
	
		// 3 or 6 hex digits, or the empty string.
		if ( preg_match('|^#([A-Fa-f0-9]{3}){1,2}$|', $color ) )
			return $color;
	
		return null;
	}
	
	/**
	 * Sanitizes a hex color without a hash. Use sanitize_hex_color() when possible.
	 *
	 * Saving hex colors without a hash puts the burden of adding the hash on the
	 * UI, which makes it difficult to use or upgrade to other color types such as
	 * rgba, hsl, rgb, and html color names.
	 *
	 * Returns either '', a 3 or 6 digit hex color (without a #), or null.
	 *
	 * @since 1.0
	 * @uses self::sanitize_hex_color()
	 *
	 * @param string $color
	 * @return string|null
	 */
	public static function sanitize_hex_color_no_hash( $color ) 
	{
		$color = ltrim( $color, '#' );
	
		if ( '' === $color )
			return '';
	
		return sanitize_hex_color( '#' . $color ) ? $color : null;
	}

	/**
	 * Current visitor/session IP address
	 *
	 * @since 1.0
	 * @return string
	 */
	public static function get_visitor_IP()
	{
		$client  = isset( $_SERVER['HTTP_CLIENT_IP'] ) ? $_SERVER['HTTP_CLIENT_IP'] : null;
		$forward = isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ? $_SERVER['HTTP_X_FORWARDED_FOR'] : null;

		if ( $client && filter_var( $client, FILTER_VALIDATE_IP ) )
			return $client;

		if ( $client && filter_var( $forward, FILTER_VALIDATE_IP ) )
			return $forward;

		return $_SERVER['REMOTE_ADDR'];
	}

	/**
	 * Determine scripts and styles enqueues suffix
	 *
	 * @since 1.0
	 * @return string
	 */
	public static function enqueue_suffix()
	{
		return defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';
	}

	/**
	 * URL Redirect 
	 * 
	 * @param string $target
	 * @param number $status
	 * @return void
	 */
	public static function redirect( $target = '', $status = 302 )
	{
		if ( '' == $target && isset( $_REQUEST['_wp_http_referer'] ) )
			$target = esc_url( $_REQUEST['_wp_http_referer'] );
	
		wp_redirect( $target, $status );
		die();
	}

	/**
	 * Modified version of sanitize_text_field with line-breaks preserved
	 *
	 * @see sanitize_text_field
	 * @since 2.9.0
	 *
	 * @param string $str
	 * @return string
	 */
	public static function sanitize_text_field_with_linebreaks( $str ) 
	{
		$filtered = wp_check_invalid_utf8( $str );

		if ( strpos($filtered, '<') !== false ) 
		{
			$filtered = wp_pre_kses_less_than( $filtered );

			// This will strip extra whitespace for us.
			$filtered = wp_strip_all_tags( $filtered, true );
		}

		$found = false;
		while ( preg_match( '/%[a-f0-9]{2}/i', $filtered, $match ) ) 
		{
			$filtered = str_replace( $match[0], '', $filtered );
			$found = true;
		}

		if ( $found ) 
		{
			// Strip out the whitespace that may now exist after removing the octets.
			$filtered = trim( preg_replace( '/ +/', ' ', $filtered ) );
		}

		/**
		 * Filter a sanitized text field string.
		 *
		 * @since 2.9.0
		 *
		 * @param string $filtered The sanitized string.
		 * @param string $str      The string prior to being sanitized.
		 */
		return apply_filters( 'sanitize_text_field_with_linebreaks', $filtered, $str );
	}

	/**
	 * Parse/Join html attributes
	 * 
	 * @param array $attrs
	 * @return string
	 */
	public static function parse_attributes( $attrs )
	{
		if ( empty( $attrs ) )
			return '';

		array_walk( $attrs, function( &$item, $key ) {
			$item = $key .'="'. esc_attr( is_array( $item ) ? implode( ' ' , $item ) : $item ) .'"';
		} );

		return implode( ' ', $attrs );
	}
}

if( !function_exists( 'dump_data_export' ) )
{
	/**
	 * Get debug information of passed data
	 * 
	 * @param mixed $data
	 * @param boolean $type
	 * @return string
	 */
	function dump_data_export( $data, $type = false )
	{
		return '<pre>'. ( $type ? var_export( $data, true ) : print_r( $data, true ) ) .'</pre>';
	}
}

if( !function_exists( 'dd' ) )
{
	/**
	 * Debug data passed and die ( exit )
	 * 
	 * @param mixed $data
	 * @param bolean $type wither to display data type or not
	 * @return void
	 */
	function dd( $data, $type = false )
	{
		dump_data( $data, $type );
		die();
	}
}

if( !function_exists( 'dump_data' ) )
{
	/**
	 * Debug data passed
	 * 
	 * @param mixed $data
	 * @param bolean $type wither to display data type or not
	 * @return void
	 */
	function dump_data( $data, $type = false )
	{
		// normal
		echo '<pre>';
		$type ? var_dump( $data ) : print_r( $data );
		echo '</pre>';
	}
}

if( !function_exists( 'multi_dump_data' ) )
{
	/**
	 * Debug multiple variables 
	 * 
	 * @param mixed $args ...
	 * @return void
	 * @uses dump_data
	 */
	function multi_dump_data( $args )
	{
		$args = func_get_args();
		foreach ( $args as $arg )
		{
			dump_data( $arg );
		}
	}
}